#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.extfile.interfaces import IExtFileContainerTarget
from pyams_content.component.links.interfaces import ILinkContainerTarget
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.component.paragraph.interfaces.video import IVideoParagraph, VIDEO_PARAGRAPH_TYPE, \
    VIDEO_PARAGRAPH_RENDERERS
from pyams_content.features.checker.interfaces import IContentChecker, MISSING_VALUE, MISSING_LANG_VALUE
from pyams_i18n.interfaces import II18nManager, INegotiator, II18n
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent

# import packages
from pyams_content.component.paragraph import BaseParagraph, BaseParagraphContentChecker, BaseParagraphFactory
from pyams_content.component.paragraph.html import check_associations
from pyams_content.features.renderer import RenderedContentMixin, IContentRenderer
from pyams_file.property import FileProperty
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config, get_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.events import subscriber
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


@implementer(IVideoParagraph, IExtFileContainerTarget, ILinkContainerTarget)
class VideoParagraph(RenderedContentMixin, BaseParagraph):
    """Video paragraph class"""

    icon_class = 'fa-film'
    icon_hint = _("Video")

    body = FieldProperty(IVideoParagraph['body'])
    description = FieldProperty(IVideoParagraph['description'])
    author = FieldProperty(IVideoParagraph['author'])
    data = FileProperty(IVideoParagraph['data'])
    renderer = FieldProperty(IVideoParagraph['renderer'])


@utility_config(name=VIDEO_PARAGRAPH_TYPE, provides=IParagraphFactory)
class VideoParagraphFactory(BaseParagraphFactory):
    """Video paragraph factory"""

    name = _("Video")
    content_type = VideoParagraph


@subscriber(IObjectAddedEvent, context_selector=IVideoParagraph)
def handle_added_video_paragraph(event):
    """Check for new associations from added paragraph"""
    paragraph = event.object
    for lang, body in (paragraph.body or {}).items():
        check_associations(paragraph, body, lang, notify=False)


@subscriber(IObjectModifiedEvent, context_selector=IVideoParagraph)
def handle_modified_video_paragraph(event):
    """Check for new associations from modified paragraph"""
    paragraph = event.object
    for lang, body in (paragraph.body or {}).items():
        check_associations(paragraph, body, lang, notify=False)


@adapter_config(context=IVideoParagraph, provides=IContentChecker)
class VideoParagraphContentChecker(BaseParagraphContentChecker):
    """Video paragraph content checker"""

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            value = i18n.get_attribute('title', lang, request)
            if not value:
                field_title = translate(IVideoParagraph['title'].title)
                if len(langs) == 1:
                    output.append(translate(MISSING_VALUE).format(field=field_title))
                else:
                    output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        for attr in ('author', 'data'):
            value = getattr(self.context, attr)
            if not value:
                output.append(translate(MISSING_VALUE).format(field=translate(IVideoParagraph[attr].title)))
        return output


@vocabulary_config(name=VIDEO_PARAGRAPH_RENDERERS)
class VideoParagraphRendererVocabulary(SimpleVocabulary):
    """Video paragraph renderers vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        if not IVideoParagraph.providedBy(context):
            context = VideoParagraph()
        terms = [SimpleTerm(name, title=translate(adapter.label))
                 for name, adapter in sorted(registry.getAdapters((context, request), IContentRenderer),
                                             key=lambda x: x[1].weight)]
        super(VideoParagraphRendererVocabulary, self).__init__(terms)
