#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphRenderer
from pyams_content.component.paragraph.interfaces.pictogram import PICTOGRAM_PARAGRAPH_TYPE, IPictogramParagraph, \
    IPictogramContainer, IPictogramContainerTarget, IPictogramItem
from pyams_content.component.paragraph.zmi import IParagraphContainerView
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons, IInnerSubForm
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_PERMISSION
from pyams_zmi.interfaces import IPropertiesEditForm
from z3c.form.interfaces import INPUT_MODE
from z3c.table.interfaces import IValues, IColumn

# import packages
from pyams_content.component.paragraph.pictogram import PictogramParagraph, PictogramItem
from pyams_content.component.paragraph.zmi import BaseParagraphAddMenu, BaseParagraphAJAXAddForm, \
    BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm
from pyams_content.features.renderer.zmi import BaseRenderedContentRenderer
from pyams_content.features.renderer.zmi.widget import RendererFieldWidget
from pyams_content.skin import pyams_content
from pyams_form.form import AJAXAddForm, AJAXEditForm
from pyams_form.security import ProtectedFormObjectMixin
from pyams_i18n.column import I18nAttrColumn
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.table import BaseTable, SorterColumn, JsActionColumn, I18nColumn, TrashColumn
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.text import get_text_start
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, InnerAdminDisplayForm, AdminDialogEditForm
from pyramid.decorator import reify
from pyramid.exceptions import NotFound
from pyramid.view import view_config
from z3c.form import field, button
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_content import _


class IPictogramsView(Interface):
    """Pictograms view marker interface"""


class IPictogramsParentForm(Interface):
    """Pictograms parent form marker interface"""


@viewlet_config(name='add-pictogram-paragraph.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=590)
class PictogramParagraphAddMenu(BaseParagraphAddMenu):
    """Pictogram paragraph add menu"""
    
    label = _("Pictograms...")
    label_css_class = 'fa fa-fw fa-linode'
    url = 'add-pictogram-paragraph.html'
    paragraph_type = PICTOGRAM_PARAGRAPH_TYPE
    
    
@pagelet_config(name='add-pictogram-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class PictogramParagraphAddForm(AdminDialogAddForm):
    """Pictogram paragraph add form"""
    
    legend = _("Add new pictogram paragraph")
    icon_css_class = 'fa fa-fw fa-linode'
    
    fields = field.Fields(IPictogramParagraph).select('title', 'renderer')
    ajax_handler = 'add-pictogram-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION
    
    def create(self, data):
        return PictogramParagraph()
    
    def add(self, object):
        IParagraphContainer(self.context).append(object)
        
        
@view_config(name='add-pictogram-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class PictogramParagraphAJAXAddForm(BaseParagraphAJAXAddForm, PictogramParagraphAddForm):
    """Pictogram paragraph add form, JSON renderer"""
    
    
@pagelet_config(name='properties.html', context=IPictogramParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class PictogramParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """Pictogram paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit pictogram paragraph properties")
    icon_css_class = 'fa fa-fw fa-linode'

    fields = field.Fields(IPictogramParagraph).select('title', 'renderer')
    fields['renderer'].widgetFactory = RendererFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION


@view_config(name='properties.json', context=IPictogramParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class PictogramParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, PictogramParagraphPropertiesEditForm):
    """Pictogram paragraph properties edit form, JSON renderer"""


@adapter_config(context=(IPictogramParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IPropertiesEditForm, IPictogramsParentForm)
class PictogramParagraphInnerEditForm(PictogramParagraphPropertiesEditForm):
    """Pictogram paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IPictogramParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class PictogramParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, PictogramParagraphInnerEditForm):
    """Pictograms paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(PictogramParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IPictogramParagraph, ())
        if 'renderer' in updated:
            form = PictogramParagraphInnerEditForm(self.context, self.request)
            form.update()
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshWidget',
                    'parent_id': '{0}_{1}_{2}'.format(
                        self.context.__class__.__name__,
                        getattr(form.getContent(), '__name__', 'noname').replace('++', ''),
                        form.id),
                    'widget_name': form.widgets['renderer'].name,
                    'content': form.widgets['renderer'].render()
                }
            })
        return output


#
# Pictogram paragraph renderer
#

@adapter_config(context=(IPictogramParagraph, IPyAMSLayer), provides=IParagraphRenderer)
class PictogramParagraphRenderer(BaseRenderedContentRenderer):
    """Pictogram paragraph renderer"""


#
# Pictogram items table view
#

class PictogramsTable(ProtectedFormObjectMixin, BaseTable):
    """Pictograms view inner table"""

    @property
    def id(self):
        return 'pictograms_{0}_list'.format(self.context.__name__)

    hide_header = True
    sortOn = None

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover', 'table-tight']
        permission = self.permission
        if (not permission) or self.request.has_permission(permission, self.context):
            classes.append('table-dnd')
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(PictogramsTable, self).data_attributes
        attributes['table'] = {
            'id': self.id,
            'data-ams-plugins': 'pyams_content',
            'data-ams-plugin-pyams_content-src': get_resource_path(pyams_content),
            'data-ams-location': absolute_url(IPictogramContainer(self.context), self.request),
            'data-ams-tablednd-drag-handle': 'td.sorter',
            'data-ams-tablednd-drop-target': 'set-pictograms-order.json'
        }
        attributes.setdefault('tr', {}).update({
            'id': lambda x, col: 'pictogram_{0}::{1}'.format(get_parent(x, IPictogramContainerTarget).__name__,
                                                             x.__name__),
            'data-ams-delete-target': 'delete-pictogram.json'
        })
        return attributes

    @reify
    def values(self):
        return list(super(PictogramsTable, self).values)


@adapter_config(context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IValues)
class PictogramsTableValuesAdapter(ContextRequestViewAdapter):
    """Pictograms table values adapter"""

    @property
    def values(self):
        return IPictogramContainer(self.context).values()


@adapter_config(name='sorter', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IColumn)
class PictogramsTableSorterColumn(ProtectedFormObjectMixin, SorterColumn):
    """Pictograms table sorter column"""


@view_config(name='set-pictograms-order.json', context=IPictogramContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_pictograms_order(request):
    """Update pictograms order"""
    order = list(map(str, json.loads(request.params.get('names'))))
    request.context.updateOrder(order)
    return {'status': 'success'}


@adapter_config(name='image', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IColumn)
class PictogramsTableImageColumn(GetAttrColumn):
    """Pictogram image column"""

    header = ''
    weight = 3

    cssClasses = {'td': 'text-center width-50'}
    dt_sortable = 'false'

    def getValue(self, obj):
        pictogram = obj.pictogram
        if pictogram is not None:
            image = II18n(pictogram).query_attribute('image', request=self.request)
            if image:
                return '<img src="{0}" />'.format(absolute_url(image, self.request, '++thumb++32x32'))
        return '--'


@adapter_config(name='show-hide', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable),
                provides=IColumn)
class PictogramsTableShowHideColumn(ProtectedFormObjectMixin, JsActionColumn):
    """Pictograms container visibility switcher column"""

    cssClasses = {'th': 'action',
                  'td': 'action switcher'}

    icon_class = 'fa fa-fw fa-eye'
    icon_hint = _("Switch pictogram visibility")

    url = 'PyAMS_content.pictograms.switchVisibility'

    weight = 5

    def get_icon(self, item):
        if item.visible:
            icon_class = 'fa fa-fw fa-eye'
        else:
            icon_class = 'fa fa-fw fa-eye-slash text-danger'
        return '<i class="{icon_class}"></i>'.format(icon_class=icon_class)

    def renderCell(self, item):
        if self.permission and not self.request.has_permission(self.permission, context=item):
            return self.get_icon(item)
        else:
            return super(PictogramsTableShowHideColumn, self).renderCell(item)


@view_config(name='set-pictogram-visibility.json', context=IPictogramContainer, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
def set_pictogram_visibility(request):
    """Set pictogram visibility"""
    container = IPictogramContainer(request.context)
    pictogram = container.get(str(request.params.get('object_name')))
    if pictogram is None:
        raise NotFound()
    pictogram.visible = not pictogram.visible
    return {'visible': pictogram.visible}


@adapter_config(name='name', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IColumn)
class PictogramsTableNameColumn(I18nColumn, I18nAttrColumn):
    """Pictograms table name column"""

    _header = _("Label")
    attrName = 'label'
    weight = 10


@adapter_config(name='body', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IColumn)
class PictogramsTableBodyColumn(I18nColumn, I18nAttrColumn):
    """Pictograms table body column"""

    _header = _("Body")
    attrName = 'body'
    weight = 20

    def getValue(self, obj):
        value = super(PictogramsTableBodyColumn, self).getValue(obj)
        if not value:
            return '--'
        return get_text_start(value, 40, 10)


@adapter_config(name='trash', context=(IPictogramContainerTarget, IPyAMSLayer, PictogramsTable), provides=IColumn)
class PictogramsTableTrashColumn(ProtectedFormObjectMixin, TrashColumn):
    """Pictograms table trash column"""


@view_config(name='delete-pictogram.json', context=IPictogramContainer, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
def delete_pictogram(request):
    """Delete pictogram"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'content': translate(_("No provided object_name argument!"))
            }
        }
    if name not in request.context:
        return {
            'status': 'message',
            'messagebox': {
                'status': 'error',
                'content': translate(_("Given association name doesn't exist!"))
            }
        }
    del request.context[name]
    return {'status': 'success'}


@adapter_config(name='pictograms', context=(IPictogramContainerTarget, IPyAMSLayer, IPictogramsParentForm),
                provides=IInnerSubForm)
@template_config(template='templates/pictograms.pt', layer=IPyAMSLayer)
@implementer(IPictogramsView)
class PictogramsView(InnerAdminDisplayForm):
    """Pictograms view"""

    fields = field.Fields(Interface)
    weight = 100

    def __init__(self, context, request, view):
        super(PictogramsView, self).__init__(context, request, view)
        self.table = PictogramsTable(context, request)
        self.table.view = self

    def update(self):
        super(PictogramsView, self).update()
        self.table.update()


#
# Pictograms forms
#

@viewlet_config(name='add-pictogram.action', context=IPictogramContainerTarget, layer=IPyAMSLayer, view=IPictogramsView,
                manager=IWidgetTitleViewletManager, permission=MANAGE_CONTENT_PERMISSION, weight=1)
class PictogramAddAction(ToolbarAction):
    """Pictogram add action"""

    label = _("Add pictogram")
    label_css_class = 'fa fa-fw fa-plus'
    url = 'add-pictogram.html'
    modal_target = True


@pagelet_config(name='add-pictogram.html', context=IPictogramContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class PictogramAddForm(AdminDialogAddForm):
    """Pictogram add form"""

    legend = _("Add new pictogram")
    icon_css_class = 'fa fa-fw fa-arrow-h'

    fields = field.Fields(IPictogramItem).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-pictogram.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(PictogramAddForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea height-100'

    def create(self, data):
        return PictogramItem()

    def add(self, object):
        IPictogramContainer(self.context).append(object)


@view_config(name='add-pictogram.json', context=IPictogramContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class PictogramAJAXAddForm(AJAXAddForm, PictogramAddForm):
    """Pictogram add form, JSON renderer"""

    def get_ajax_output(self, changes):
        table = PictogramsTable(self.context, self.request)
        table.update()
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Pictogram was correctly added")),
            'events': [{
                'event': 'myams.refresh',
                'options': {
                    'handler': 'PyAMS_content.pictograms.refreshPictograms',
                    'object_id': table.id,
                    'table': table.render()
                }
            }]
        }


@pagelet_config(name='properties.html', context=IPictogramItem, layer=IPyAMSLayer, permission=MANAGE_CONTENT_PERMISSION)
class PictogramPropertiesEditForm(AdminDialogEditForm):
    """Pictogram properties edit form"""

    legend = _("Edit pictogram properties")
    icon_css_class = 'fa fa-fw fa-linode'

    fields = field.Fields(IPictogramItem).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(PictogramPropertiesEditForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea height-100'


@view_config(name='properties.json', context=IPictogramItem, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class PictogramPropertiesAJAXEditForm(AJAXEditForm, PictogramPropertiesEditForm):
    """Pictogram properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(PictogramPropertiesAJAXEditForm, self).get_ajax_output(changes)
        updated = changes.get(IPictogramItem, ())
        if updated:
            target = get_parent(self.context, IPictogramContainerTarget)
            table = PictogramsTable(target, self.request)
            table.update()
            row = table.setUpRow(self.context)
            output.setdefault('events', []).append({
                'event': 'myams.refresh',
                'options': {
                    'handler': 'MyAMS.skin.refreshRow',
                    'object_id': 'pictogram_{0}::{1}'.format(target.__name__,
                                                             self.context.__name__),
                    'row': table.renderRow(row)
                }
            })
        return output
